searchData={"items":[{"type":"module","title":"Memoize","doc":"Module documentation for Memoize.","ref":"Memoize.html"},{"type":"function","title":"Memoize.cache_strategy/0","doc":"","ref":"Memoize.html#cache_strategy/0"},{"type":"macro","title":"Memoize.defmemo/2","doc":"Define the memoized function.\n\nBelow code:\n\n    defmemo foo(0, y) do\n      y\n    end\n\n    defmemo foo(x, y) when x == 1 do\n      y * z\n    end\n\n    defmemo foo(x, y, z \\\\ 0) when x == 2 do\n      y * z\n    end\n\nis converted to:\n\n    def foo(t1, t2) do\n      Memoize.Cache.get_or_run({__MODULE__, :foo, [t1, t2]}, fn -> __foo_memoize(t1, t2) end)\n    end\n\n    def foo(t1, t2, t3) do\n      Memoize.Cache.get_or_run({__MODULE__, :foo, [t1, t2, t3]}, fn -> __foo_memoize(t1, t2, t3) end)\n    end\n\n    def __foo_memoize(0, y) do\n      y\n    end\n\n    def __foo_memoize(x, y) when x == 1 do\n      y * z\n    end\n\n    def __foo_memoize(x, y, z \\\\ 0) when x == 2 do\n      y * z\n    end","ref":"Memoize.html#defmemo/2"},{"type":"macro","title":"Memoize.defmemo/3","doc":"","ref":"Memoize.html#defmemo/3"},{"type":"macro","title":"Memoize.defmemop/2","doc":"","ref":"Memoize.html#defmemop/2"},{"type":"macro","title":"Memoize.defmemop/3","doc":"","ref":"Memoize.html#defmemop/3"},{"type":"function","title":"Memoize.garbage_collect/0","doc":"","ref":"Memoize.html#garbage_collect/0"},{"type":"function","title":"Memoize.invalidate/0","doc":"","ref":"Memoize.html#invalidate/0"},{"type":"function","title":"Memoize.invalidate/1","doc":"","ref":"Memoize.html#invalidate/1"},{"type":"function","title":"Memoize.invalidate/2","doc":"","ref":"Memoize.html#invalidate/2"},{"type":"function","title":"Memoize.invalidate/3","doc":"","ref":"Memoize.html#invalidate/3"},{"type":"module","title":"Memoize.Cache","doc":"Module documentation for Memoize.Cache.","ref":"Memoize.Cache.html"},{"type":"function","title":"Memoize.Cache.garbage_collect/0","doc":"","ref":"Memoize.Cache.html#garbage_collect/0"},{"type":"function","title":"Memoize.Cache.get_or_run/3","doc":"","ref":"Memoize.Cache.html#get_or_run/3"},{"type":"function","title":"Memoize.Cache.invalidate/0","doc":"","ref":"Memoize.Cache.html#invalidate/0"},{"type":"function","title":"Memoize.Cache.invalidate/1","doc":"","ref":"Memoize.Cache.html#invalidate/1"},{"type":"behaviour","title":"Memoize.CacheStrategy","doc":"A behaviour module for implementing cache strategy.","ref":"Memoize.CacheStrategy.html"},{"type":"callback","title":"Memoize.CacheStrategy.cache/3","doc":"","ref":"Memoize.CacheStrategy.html#c:cache/3"},{"type":"function","title":"Memoize.CacheStrategy.configured?/1","doc":"","ref":"Memoize.CacheStrategy.html#configured?/1"},{"type":"callback","title":"Memoize.CacheStrategy.garbage_collect/0","doc":"","ref":"Memoize.CacheStrategy.html#c:garbage_collect/0"},{"type":"callback","title":"Memoize.CacheStrategy.init/0","doc":"","ref":"Memoize.CacheStrategy.html#c:init/0"},{"type":"callback","title":"Memoize.CacheStrategy.init/1","doc":"","ref":"Memoize.CacheStrategy.html#c:init/1"},{"type":"callback","title":"Memoize.CacheStrategy.invalidate/0","doc":"","ref":"Memoize.CacheStrategy.html#c:invalidate/0"},{"type":"callback","title":"Memoize.CacheStrategy.invalidate/1","doc":"","ref":"Memoize.CacheStrategy.html#c:invalidate/1"},{"type":"callback","title":"Memoize.CacheStrategy.read/3","doc":"","ref":"Memoize.CacheStrategy.html#c:read/3"},{"type":"callback","title":"Memoize.CacheStrategy.tab/1","doc":"","ref":"Memoize.CacheStrategy.html#c:tab/1"},{"type":"module","title":"Memoize.CacheStrategy.Eviction","doc":"","ref":"Memoize.CacheStrategy.Eviction.html"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.cache/3","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#cache/3"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.clear_expired_cache/2","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#clear_expired_cache/2"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.garbage_collect/0","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#garbage_collect/0"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.init/1","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#init/1"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.invalidate/0","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#invalidate/0"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.invalidate/1","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#invalidate/1"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.read/3","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#read/3"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.tab/1","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#tab/1"},{"type":"function","title":"Memoize.CacheStrategy.Eviction.used_bytes/0","doc":"","ref":"Memoize.CacheStrategy.Eviction.html#used_bytes/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"develop - Changelog","doc":"","ref":"changelog.html#develop"},{"type":"extras","title":"1.4.3 2023-12-23 - Changelog","doc":"- Update dependencies\n- Update supported Elixir version from 1.9 to 1.11","ref":"changelog.html#1-4-3-2023-12-23"},{"type":"extras","title":"1.4.2 2023-06-22 - Changelog","doc":"- Update dependencies","ref":"changelog.html#1-4-2-2023-06-22"},{"type":"extras","title":"1.4.1 2022-09-04 - Changelog","doc":"- Update dependencies\n- fix: Invalidate cache with map\n    - @phanmn","ref":"changelog.html#1-4-1-2022-09-04"},{"type":"extras","title":"1.4.0 2021-07-30 - Changelog","doc":"- Add the default value of `:expires_in` configurable.\n- Remove compile-time config.","ref":"changelog.html#1-4-0-2021-07-30"},{"type":"extras","title":"1.3.3 2021-02-03 - Changelog","doc":"- Fix infinite loop if cache process crashes ([#14](https://github.com/melpon/memoize/pull/14))\n    - @davorbadrov","ref":"changelog.html#1-3-3-2021-02-03"},{"type":"extras","title":"1.3.2 2020-10-09 - Changelog","doc":"- Replace `is_exception/1` to `Exception.exception?/1`.","ref":"changelog.html#1-3-2-2020-10-09"},{"type":"extras","title":"1.3.1 2020-10-07 - Changelog","doc":"- Require Elixir >= 1.9\n- Fix warnings for Elixir 1.11\n- Fix freeze when exit or throw ([#13](https://github.com/melpon/memoize/issues/13))\n- Update dependencies","ref":"changelog.html#1-3-1-2020-10-07"},{"type":"extras","title":"1.3.0 2018-10-31 - Changelog","doc":"- Limit count of waiter processes that receive message passing\n- Update dependencies","ref":"changelog.html#1-3-0-2018-10-31"},{"type":"extras","title":"1.2.8 2018-07-21 - Changelog","doc":"- Fix `defmemo unquote(name)()` doesn't work","ref":"changelog.html#1-2-8-2018-07-21"},{"type":"extras","title":"1.2.7 2018-04-16 - Changelog","doc":"- Update dependencies","ref":"changelog.html#1-2-7-2018-04-16"},{"type":"extras","title":"1.2.6 2018-01-23 - Changelog","doc":"- Apply Elixir formatter","ref":"changelog.html#1-2-6-2018-01-23"},{"type":"extras","title":"1.2.5 2018-01-06 - Changelog","doc":"- Update a dependency `ex_doc`\n- Improve documents","ref":"changelog.html#1-2-5-2018-01-06"},{"type":"extras","title":"1.2.4 2017-10-22 - Changelog","doc":"- Fix map type is passed to `:ets.select_replace/2`\n- Improve documents","ref":"changelog.html#1-2-4-2017-10-22"},{"type":"extras","title":"1.2.3 2017-09-28 - Changelog","doc":"- Fix passing map type to `:ets.select_replace/2`","ref":"changelog.html#1-2-3-2017-09-28"},{"type":"extras","title":"Memoize","doc":"# Memoize\n\n[![Module Version](https://img.shields.io/hexpm/v/memoize.svg)](https://hex.pm/packages/memoize)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/memoize/)\n[![Total Download](https://img.shields.io/hexpm/dt/memoize.svg)](https://hex.pm/packages/memoize)\n[![License](https://img.shields.io/hexpm/l/memoize.svg)](https://github.com/melpon/memoize/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/melpon/memoize.svg)](https://github.com/melpon/memoize/commits/master)\n\nA memoization macro for Elixir.\n\n> \"In computing, memoization or memoisation is an optimization technique used\n> primarily to speed up computer programs by storing the results of expensive\n> function calls and returning the cached result when the same inputs occur\n> again.\"\n>\n> Source: https://en.wikipedia.org/wiki/Memoization","ref":"readme.html"},{"type":"extras","title":"Requirement - Memoize","doc":"- Elixir 1.9 or later.\n- Erlang/OTP 21.2 or later.","ref":"readme.html#requirement"},{"type":"extras","title":"Installation - Memoize","doc":"Add `:memoize` to your `mix.exs` dependencies:\n\n```elixir\ndefp deps do\n  [\n    {:memoize, \"~> 1.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"How to memoize - Memoize","doc":"If you want to cache a function, `use Memoize` on the module and change `def` to `defmemo`.\n\nFor example:\n\n```elixir\ndefmodule Fib do\n  def fibs(0), do: 0\n  def fibs(1), do: 1\n  def fibs(n), do: fibs(n - 1) + fibs(n - 2)\nend\n```\n\nThis code changes to:\n\n```elixir\ndefmodule Fib do\n  use Memoize\n  defmemo fibs(0), do: 0\n  defmemo fibs(1), do: 1\n  defmemo fibs(n), do: fibs(n - 1) + fibs(n - 2)\nend\n```\n\nIf a function defined by `defmemo` raises an error, the result is not cached and one of waiting processes will call the function.","ref":"readme.html#how-to-memoize"},{"type":"extras","title":"Exclusive - Memoize","doc":"A caching function that is defined by `defmemo` is never called in parallel.\n\n```elixir\ndefmodule Calc do\n  use Memoize\n  defmemo calc() do\n    Process.sleep(1000)\n    IO.puts \"called!\"\n  end\nend\n\n# call `Calc.calc/0` in parallel using many processes.\nfor _ <- 1..10000 do\n  Process.spawn(fn -> Calc.calc() end, [])\nend\n\n# but, actually `Calc.calc/0` is called only once.\n```","ref":"readme.html#exclusive"},{"type":"extras","title":"Invalidate - Memoize","doc":"If you want to invalidate cache, you can use `Memoize.invalidate/{0-3}`.\n\n```elixir\n# invalidate a cached value of `Fib.fibs(0)`.\nMemoize.invalidate(Fib, :fibs, [0])\n\n# invalidate all cached values of `Fib.fibs/1`.\nMemoize.invalidate(Fib, :fibs)\n\n# invalidate all cached values of `Fib` module.\nMemoize.invalidate(Fib)\n\n# invalidate all cached values.\nMemoize.invalidate()\n```\n\nNotice: `Memoize.invalidate/{0-2}`'s complexity is linear. Therefore, it takes a long time if `Memoize` has many cached values.","ref":"readme.html#invalidate"},{"type":"extras","title":"Caching Partial Arguments - Memoize","doc":"If you want to cache with partial arguments, use `Memoize.Cache.get_or_run/2` directly.\n\n```elixir\ndefmodule Converter do\n  def convert(unique_key, data) do\n    Memoize.Cache.get_or_run({__MODULE__, :resolve, [unique_key]}, fn ->\n      do_convert(data)\n    end)\n  end\nend\n```","ref":"readme.html#caching-partial-arguments"},{"type":"extras","title":"Cache Strategy - Memoize","doc":"Cache strategy is a behaviour to management cached values.\n\nBy default, the caching strategy is `Memoize.CacheStrategy.Default`.\n\nIf you want to change the caching strategy, configure `:cache_strategy` in `:memoize` application.\n\n```elixir\nconfig :memoize,\n  cache_strategy: Memoize.CacheStrategy.Eviction\n```\n\n`memoize` provides below caching strategies.\n\n- `Memoize.CacheStrategy.Default`\n- `Memoize.CacheStrategy.Eviction`","ref":"readme.html#cache-strategy"},{"type":"extras","title":"Cache Strategy - Memoize.CacheStrategy.Default - Memoize","doc":"Default caching strategy.\nIt provides only simple and fast features.\n\nBasically, cached values are not collected automatically.\nTo collect cached values, call `invalidate/{0-4}`, call `garbage_collect/0` or specify `:expires_in` with `defmemo`.\n\n#","ref":"readme.html#cache-strategy-memoize-cachestrategy-default"},{"type":"extras","title":"Expiration - Memoize","doc":"If you want to invalidate the cache after a certain period of time, you can use `:expires_in`.\n\n```elixir\ndefmodule Api do\n  use Memoize\n  defmemo get_config(), expires_in: 60 * 1000 do\n    call_external_api()\n  end\nend\n```\n\nThe cached value is invalidated in the first `get_config/0` function call after `expires_in` milliseconds have elapsed.\n\nTo collect expired values, you can use `garbage_collect/0`. It collects all expired values. Its complexity is linear.\n\nThe default value of `:expires_in` is configurable as below:\n\n```elixir\nconfig :memoize,\n  cache_strategy: Memoize.CacheStrategy.Default\n\nconfig :memoize, Memoize.CacheStrategy.Default,\n  expires_in: 600_000 # 10 minutes\n```","ref":"readme.html#expiration"},{"type":"extras","title":"Cache Strategy - Memoize.CacheStrategy.Eviction - Memoize","doc":"`Memoize.CacheStrategy.Eviction` is one of caching strategy.\nIt provides many features, but slower than `Memoize.CacheStrategy.Default`.\n\nThe strategy is, basically, if cached memory size is exceeded `max_threshold`, *unused* cached values are collected until memory size falls below `min_threshold`.\n\nTo use `Memoize.CacheStrategy.Eviction`, configure `:cache_strategy` as below:\n\n```elixir\nconfig :memoize,\n  cache_strategy: Memoize.CacheStrategy.Eviction\n\nconfig :memoize, Memoize.CacheStrategy.Eviction,\n  min_threshold: 5_000_000,\n  max_threshold: 10_000_000\n```\n\n#","ref":"readme.html#cache-strategy-memoize-cachestrategy-eviction"},{"type":"extras","title":"Permanently - Memoize","doc":"If `:permanent` option is specified with `defmemo`, the value won't be collected automatically.\nIf you want to remove the value, call `invalidate/{0-3}`.\n\n```elixir\ndefmodule Json do\n  use Memoize\n  defmemo get_json(filename), permanent: true do\n    filename |> File.read!() |> Poison.decode!()\n  end\nend\n```\n\nNotice the permanent value includes in used memory size. So you should adjust `min_threshold` value.\n\n#","ref":"readme.html#permanently"},{"type":"extras","title":"Expiration - Memoize","doc":"If `:expires_in` option is specified with `defmemo`, the value will be collected after `:expires_in` milliseconds.\nTo be exact, when the `read/3` function is called with any arguments, all expired values will be collected.\n\n```elixir\ndefmodule Api do\n  use Memoize\n  defmemo get_config(), expires_in: 60 * 1000 do\n    call_external_api()\n  end\nend\n```\n\nYou can both specify `:permanent` and `:expires_in`.\nIn the case, the cached value is not collected by `garbage_collect/0` or memory size that exceed `max_threshold`, but after `:expires_in` milliseconds it is collected.","ref":"readme.html#expiration"},{"type":"extras","title":"Cache Strategy - Your Strategy - Memoize","doc":"You can customize caching strategy.\n\n```elixir\ndefmodule Memoize.CacheStrategy do\n  @callback init() :: any\n  @callback tab(any) :: atom\n  @callback cache(any, any, Keyword.t) :: any\n  @callback read(any, any, any) :: :ok | :retry\n  @callback invalidate() :: integer\n  @callback invalidate(any) :: integer\n  @callback garbage_collect() :: integer\nend\n```\n\nIf you want to use a customized caching strategy, implement `Memoize.CacheStrategy` behaviour.\n\n```elixir\ndefmodule YourAwesomeApp.ExcellentCacheStrategy do\n  @behaviour Memoize.CacheStrategy\n\n  def init() do\n    ...\n  end\n\n  ...\nend\n```\n\nThen, configure `:cache_strategy` in `:memoize` application.\n\n```elixir\nconfig :memoize,\n  cache_strategy: YourAwesomeApp.ExcellentCacheStrategy\n```\n\nNotice `tab/1`, `read/3`, `invalidate/{0-1}`, `garbage_collect/0` are called concurrently.\n`cache/3` is not called concurrently, but other functions are called concurrently while `cache/3` is called by a process.\n\n#","ref":"readme.html#cache-strategy-your-strategy"},{"type":"extras","title":"init/0 - Memoize","doc":"When application is started, `init/0` is called only once.\n\n#","ref":"readme.html#init-0"},{"type":"extras","title":"tab/1 - Memoize","doc":"To determine which ETS tab to use, Memoize calls `tab/0`.\n\n#","ref":"readme.html#tab-1"},{"type":"extras","title":"cache/3 - Memoize","doc":"When new value is cached, `cache/3` will be called.\nThe first argument is `key` that is used as cache key.\nThe second argument is `value` that is calculated value by cache key.\nThe third argument is `opts` that is passed by `defmemo`.\n\n`cache/3` can return an any value that is called `context`.\n`context` is stored to ETS.\nAnd then, the context is passed to `read/3`'s third argument.\n\n#","ref":"readme.html#cache-3"},{"type":"extras","title":"read/3 - Memoize","doc":"When a value is looked up by a key, `read/3` will be called.\nfirst and second arguments are same as `cache/3`.\nThe third argument is `context` that is created at `cache/3`.\n\n`read/3` can return `:retry` or `:ok`.\nIf `:retry` is returned, retry the lookup.\nIf `:ok` is returned, return the `value`.\n\n#","ref":"readme.html#read-3"},{"type":"extras","title":"invalidate/{0,1} - Memoize","doc":"These functions are called from `Memoize.invalidate/{0-4}`.\n\n#","ref":"readme.html#invalidate-0-1"},{"type":"extras","title":"garbage_collect/0 - Memoize","doc":"The function is called from `Memoize.garbage_collect/0`.","ref":"readme.html#garbage_collect-0"},{"type":"extras","title":"Waiter config - Memoize","doc":"Normally, waiter processes are waiting at the end of the computing process using message passing. However, As the number of waiting processes increases, memory is consumed, so we limit this number of the waiters.\n\nNumber of waiter processes receiving message passing are configured as `config.exs` or `defmemo` opts. (prior `defmemo`).\n\nWith `config.exs`:\n\n```elixir\nconfig :memoize,\n  max_waiter: 100,\n  waiter_sleep_ms: 1000\n```\n\nWith `defmemo` opts:\n\n```elixir\ndefmemo foo(), max_waiter: 100, waiter_sleep_ms: 1000 do\n  ...\nend\n```\n\n- `:max_waiters`: Number of waiter processes receiving message passing. (default: 20)\n- `:waiter_sleep_ms`: Time to sleep when the number of waiter processes exceeds `:max_waiters`. (default: 200)","ref":"readme.html#waiter-config"},{"type":"extras","title":"Internal - Memoize","doc":"`Memoize` is using CAS (compare-and-swap) on ETS.\n\nCAS is [now available](http://erlang.org/doc/man/ets.html#select_replace-2) in Erlang/OTP 20.","ref":"readme.html#internal"},{"type":"extras","title":"License - Memoize","doc":"Copyright (c) 2017 melpon\n\nThis library is MIT licensed. See the [LICENSE](https://github.com/melpon/memoize/blob/master/LICENSE)\nfor details.","ref":"readme.html#license"}],"content_type":"text/markdown"}